#Script that processes the raw data and time merges the seperate files from the spectral and pH recordings
library("xlsx")

setwd("Your working directory")

#Routine that reads the spectral files and extracts the desired wavelengths
# 1: Load O2 binding curves ----------

#reading files from data directory
setwd("./data")
files = list.files(pattern="^[0-9](.*)[0-9][0-9].txt$") #lists all the files starting with O and ending with a number and .txt (^ indicates the start, $ the end, and (.*) anything in between

#empty dataframe that will hold the raw data
hcdata=data.frame(File =NULL, Date = NULL, Time = NULL, WavelengthHc = NULL, ExtinctionHc = NULL, WavelenghtProt = NULL, ExtinctionProt = NULL) 

#loop that reads each file and calculates the extinction maxima
#i = 1
for(i in seq(along=files)) {
 
  hcraw = read.table(file=files[i], skip=17, strip.white = TRUE, comment.char = ">") #comment.char =">" tells R what lines are comments; strip white deletes extra spaces before and after the character variables 
  
  #extraction of times and date
  date = scan(file = files[i], what = character(), nmax=7, skip = 2, quiet=TRUE) #reads the date line of the text file as a list, nmax defines the maximum number of values to be read, skip = 2 skips the first two lines, quit supresses the message how many lines were read
  time = date[5]
  date = paste(date[4], date[3], date[7]) #combines strings from the list to generate a date variable
  
  #smooth fit to reduce noise
  hcfit = data.frame(loess.smooth(hcraw[c(350:570),1],hcraw[c(350:570),2], span = 0.03, degree = 1,family = c("gaussian"))) #smothes noisy data according to optimised span after Loess
  
  #extraction of the protein peak
  protfit = data.frame(loess.smooth(hcraw[c(270:340),1],hcraw[c(270:340),2], span = 0.3, degree = 1,family = c("gaussian"))) #smothes noisy data according to optimised span after Loess
  maxprot = max(protfit$y) #finds the maximum of the smoothed protein peak
  
  #extraction of the oxygenation peak
  RowHc = which(hcfit$x>347.5 & hcfit$x<348.5) # Row index of the absorbance value that is closest to 348nm
  ODHc = hcfit[RowHc,2]                        # saves absorbance (OD) value at 348nm
  wlhc = hcfit[RowHc,1]                        # saves wavelength that is closest to 348nm
  
  RowProt = which(protfit$y == maxprot)    # Row index of the maximum protein absorbance value
  wlprot = protfit[RowProt,1]              # saves wavelength at which protein absorbance is maximal
  
  #excludes mutiple maximum or minimum values caused by overload measurements
  if (length(wlhc)!=1) ODhc = 0  
  if (length(wlprot)!=1) maxprot = ""
  if (length(wlhc)!=1) wlhc = "" 
  if (length(wlprot)!=1) wlprot = ""
  
  #plots the raw data
  hcfit_curve = data.frame(loess.smooth(hcraw[c(245:2048),1],hcraw[c(245:2048),2], span = 0.05, degree = 1,family = c("gaussian"), evaluation = 825)) 
  par(new = T)
  plot(hcfit_curve, type = "l", ylim = c(-0.1,1.2), xlim = c(260,900), bty = "L", col = rgb(1,26,188,100, maxColorValue=255), xlab = "Wavelength (nm)", ylab = "Extinction (A)")
  
  #combines all required data in one data frame
  hc = data.frame(date, time, wlhc,ODHc, wlprot, maxprot)	#creates data frame with the results
  hc = data.frame(file=rep(files[i], length(hc[,1])), hc) # adds file tracking column to variable
  
  hcdata = rbind(hcdata, hc)	# appends data from all files to data frame 'hcdata'
  print(files[i])
}

#saves the plot of all spectra as pdf
dev.copy2pdf(file = paste("../data.pdf", sep=""), width = 20, height = 10)
dev.off

# 2: Loading pH data ----

#finds the file that contains the pH data from the phase detection device
files = list.files(pattern="^[pP][hH](.*)[0-9].txt") 
pHraw = read.table(file= files[1], sep= ";", dec=".", header=TRUE, skip = 57, strip.white = TRUE)[c(200:290),]

#Replacing overshoot/0 pH values with blanks
zero = which(pHraw[,4]==0)
pHraw[zero,4] = ""

pH = pHraw[,c(1:7)]

# 3: Matches Hc and pH data tables via time ---- 
library(data.table)

pH1 = pH
hcdata1 = hcdata
#conversion from factor to time formats
pH1$time.hh.mm.ss = as.ITime(pH1$time.hh.mm.ss, format = "%H:%M:%S") 
hcdata1$time = as.ITime(hcdata1$time, format = "%H:%M:%S")           

# creates empty data frame to store data from loop
pHdata=data.frame(matrix(nrow=0, ncol=0)) 

#loop that matches the spectral and pH data by time
for (i in 1:nrow(pH1)){
  
  timediff = difftime(pH1[i,2], hcdata1$time)                                        #calculates time differences of whole column
  row = which.min(abs(timediff))                                                     #gets row index of smallest timedifference
  
  pHdata = rbind(pHdata, data.frame(hcdata1[row,], pH[i,]))                          #matches closest times and binds it to a data frame
}


BlankRows = which(pHdata$pH=="")                                #finds rows where pH values are empty due to signal overshoots
if (length(BlankRows) != 0) {pHdata2 = pHdata[c(-BlankRows),]}  #delete rows with missing pH values if there are any missing values

pHdata$time.hh.mm.ss = as.ITime(pHdata$time.hh.mm.ss, format = "%H:%M:%S")         #converts factors to time format

timediff = difftime(pHdata$time,pHdata$time.hh.mm.ss)                              #calculates difference between times of pH and absorbance recordings
timediff = abs(timediff)                                                           #gets absolute values of time differences

#final data frame saved as excel file
HcdataComb = cbind(pHdata, timediff) 
HcdataComb = HcdataComb[,-8] #deletes duplicate date column
write.xlsx2(HcdataComb, file=paste("../Hcdata.xlsx", sep = ""), sheetName="Hcdata")

